/**
 * @file    reHDD.h
 * @brief   represent
 * @author  hendrik schutter
 * @date    01.05.2020
 */

#ifndef REHDD_H_
#define REHDD_H_

#define DRYRUN

#define WORSE_HOURS   19200 //mark drive if at this limit or beyond
#define WORSE_POWERUP 4000  //mark drive if at this limit or beyond

#define SHRED_ITERATIONS 1

#define SELECTED_DRIVE vecDrives.at(i32SelectedEntry)

#define READ 0
#define WRITE 1

#include <iostream>
#include <string>
#include <fstream>
#include <tuple>
#include <vector>
#include <time.h>
#include <chrono>
#include <curses.h>
#include <thread>
#include <unistd.h>
#include <mutex>
#include <sys/select.h>
#include <algorithm>
#include <cstring>
#include <sstream>
#include <iomanip>
#include <signal.h>

using namespace std;

#include "drive.h"
#include "smart.h"
#include "shred/shred.h"
#include "delete.h"
#include "tui.h"


template <typename T, typename I> T* iterator_to_pointer(I i)
{
    return (&(*i));
}

class reHDD
{
protected:

public:
    reHDD(void);
    void app_logic();

private:

    static void searchDrives(vector <Drive>* pvecDrives);
    static void printDrives(vector <Drive>* pvecDrives);
    static  void filterIgnoredDrives(vector <Drive>* pvecDrives);
    static void filterNewDrives(vector <Drive>* pvecOldDrives, vector <Drive>* pvecNewDrives);
    static void addSMARTData(vector <Drive>* pvecDrives);
    static void ThreadScannDevices();
    static void ThreadUserInput();
    static void ThreadShred();
    static void handleArrowKey(TUI::UserInput userInput);
    static void handleEnter();
    static void handleESC();
    static void handleAbort();
    static void checkShredComplete(vector <Drive>* pvecDrives);


};


#endif  // REHDD_H_